/*  plumb - the pipe shell - VP defaults
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

/*******************************/
/* virtual process defaults    */
/* sets default event handlers */
/*******************************/
#include <stdlib.h>
#include "process.h"
#include "pipe.h"
#include "plumb.h"
#include "virt.h"

virt_ctrl_t vp_default_input(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	plumb_error("VP didn't bind input()\n");
	return VC_QUIT; /* won't reach this, supress compiler warning */
}

/* eof on sink (input): pass on eof to all sources and exit */
virt_ctrl_t vp_default_sink_eof(proc_virt_t *proc, int local_fd)
{
	int n;

	for(n = 0; n < proc->h.num_fds; n++)
		if (is_source(&proc->h.fds[n]))
			fd_close(&proc->h.fds[n]);

	return VC_QUIT;
}

/* eof on source (output): pass on eof to all sinks and exit */
virt_ctrl_t vp_default_source_eof(proc_virt_t *proc, int local_fd)
{
	int n;

	for(n = 0; n < proc->h.num_fds; n++)
		if (is_sink(&proc->h.fds[n]))
			fd_close(&proc->h.fds[n]);

	return VC_QUIT;
}

/* block on a source (output): block all sinks */
virt_ctrl_t vp_default_source_block(proc_virt_t *proc, int local_fd, int is_blocking)
{
	int n;

	for(n = 0; n < proc->h.num_fds; n++)
		if (is_sink(&proc->h.fds[n]))
			back_block(&proc->h.fds[n], is_blocking);

	return VC_OK;
}

/* discard data */
virt_ctrl_t vp_input_discard(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	return VC_OK;
}


virt_ctrl_t vp_default_newfd(proc_virt_t *proc, int local_fd)
{
	return vp_stdfd_enforce(proc, local_fd, 1);
}

virt_ctrl_t vp_default_delfd(proc_virt_t *proc, int local_fd)
{
	return vp_stdfd_enforce(proc, local_fd, 0);
}

void vp_default(proc_virt_t *proc)
{
	proc->input        = vp_default_input;
	proc->sink_eof     = vp_default_sink_eof;
	proc->source_eof   = vp_default_source_eof;
	proc->source_block = vp_default_source_block;
	proc->timer        = NULL;                     /* ignore timer events by default */
	proc->newfd        = vp_default_newfd;         /* enforce stdfd */
	proc->delfd        = vp_default_delfd;         /* enforce stdfd */
	proc->start        = NULL;                     /* ignore start - nothing to look at or allocate */
	proc->stop         = NULL;                     /* ignore stop - nothing to clean up */
	proc->data = NULL;
}

virt_ctrl_t vp_input_copy(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	vp_write(proc, vstdout, data, len);
	return VC_OK;
}


virt_ctrl_t vp_eof_shadowed_panic(proc_virt_t *proc, int local_fd)
{
	plumb_error("Direct fd call (eof on %d) to VP %s=[%s] while it is using a shadow RP.\n", local_fd, proc->h.id, proc->cmd);
	return VC_QUIT;
}

virt_ctrl_t vp_input_shadowed_panic(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	plumb_error("Direct fd call (input on %d) to VP %s=[%s] while it is using a shadow RP.\n", local_fd, proc->h.id, proc->cmd);
	return VC_QUIT;
}
